#ifndef MAX_ROBOTS
/*
 *	eek.h - eek stuff
 *
 *	Copyright 1988, 1990, Miles ONeal, Austin, TX, USA. All rights reserved.
 *	Copyright 1993, 1997-8 Roadkills-R-Us, Austin, TX, USA. All rights reserved.
 */
#include <stdio.h>
#include <curses.h>
#include <signal.h>
#include <sys/types.h>

#ifndef __CEXTRACT__
#	include "eek_proto.h"
#endif
#include <Menus.h>
#include <Curses.h>
#include <global.h>
#include <general.h>

GLOBAL char *eek_version INIT1 ( EEK_VERSION );
/*
 *	minimum screen size to play eek (& not much fun at that!)
 */
#define MIN_LINES	24
#define MIN_COLS	44
/*
 *	real playing field size
 */
#define INX				60
#define INY				24

GLOBAL int NX	INIT1 (INX);
GLOBAL int NY	INIT1 (INY);
GLOBAL int SY	INIT1 (22);	/* score Y loc */
/*
 *	maximum size we'll let field get
 */
#define MAXX			132
#define MAXY			66
/*
 *	who the players is
 */
#define HUMANOID		(chtype) 'I'
#define ROBOT			(chtype) '='
#define FENCE			(chtype) '#'
#define JUNK			(chtype) '@'
#define BERSERKER		(chtype) 'X'
#define TRAP			(chtype) '^'
#define JUGGERNAUT	(chtype) '*'
#define BLACK_HOLE	(chtype) '.'
/*
 *	what killed you when game's over
 */
GLOBAL char robot_obit[] INIT1 ({"turned into a zombie"});
GLOBAL char fence_obit[] INIT1 ({"died on a fence"});
GLOBAL char junk_obit[] INIT1 ({"zapped in a junk heap"});
GLOBAL char berserker_obit[] INIT1 ({"went slowly insane"});
GLOBAL char trap_obit[] INIT1 ({"died like a berserker"});
GLOBAL char juggernaut_obit[] INIT1 ({"was smashed flat"});
GLOBAL char bh_obit[] INIT1 ({"disappeared!"});
GLOBAL char unknown_soldier[] INIT1 ({"died mysteriously"});
GLOBAL char *user_name INIT1 ("");
/*
 *	black hole gravity well effect
 */
#define NO_BH			0		/* no black hole */
#define LITTLE_BH		1		/* little black hole */
#define BIG_BH			2		/* big black hole */
/*
 *	reasons for leaving
 */
#define SURVIVED		0		/* my hero! */
#define KILLED			1		/* painful death */
#define WIMPED_OUT	2		/* quit */
#define RETREATED		3		/* saved game */
#define ESCAPED		4		/* emergency bailout */
/*
 *	health possibilities
 */
#define ALIVE			1
#define DEAD 			0
/*
 *	various initializations
 */
#define MIN_ROBOTS	12
#define MAX_ROBOTS	100
#define MIN_FENCES	2
#define MAX_FENCES	8
#define MAX_TRAPS		((nberserk>3) ? ((nberserk)/2) : (nberserk))
#define MIN_SURV_SCORE	30

#define JUG_LIKELY	4		/* 1/JUG_LIKELY chance of a juggernaut */
#define NICE_LVL		10		/* 1/NICE_LVL chance we won't check teleport */

#ifndef EEKDIR
  GLOBAL char *eek_dir	INIT1 ("/usr/games/lib/eekdir");
#else /* EEKDIR */
  GLOBAL char *eek_dir	INIT1 ( EEKDIR );
#endif /* EEKDIR */

#ifndef TRAINING
#  define TRAINING "teek"
#endif /* TRAINING */

GLOBAL int
	speed INIT1(0),						/* game speed (def = 0 (waits 4 you)) */
	wait_time INIT1(0),					/* time to wait between moves */
	madman INIT1 (0),						/* madman test mode */
	need_box INIT1 (1),					/* need to redraw playing field bounds? */
	level INIT1 (0),						/* what IS this level */
	new_level INIT1(0),					/* 1 if level just changed, else 0 */
	moves INIT1 (0),						/* # moves so far this level */

	nice_lvl INIT1 (NICE_LVL),			/* teleportation assistance likelihood */
	nice_dist INIT1 (1),					/* teleportation closeness check */
	tele_tries INIT1 (0),				/* # tries at being nice */

	health INIT1 (ALIVE),				/* ALIVE or DEAD */
	lives INIT1 (0),						/* any extra lives? */

	nrobots INIT1 (MIN_ROBOTS),		/* # robots to start with */
	maxrobots INIT1 (MAX_ROBOTS),		/* max # robots on a level */
	rob_pts INIT1 (10),					/* what's a robots worth? */

	nfences INIT1 (MAX_FENCES),		/* # fences to start with */
	minfences INIT1 (MIN_FENCES),		/* min # fences on a level */

	nberserk INIT1 (0),					/* # berserkers this level */
	ber_pts INIT1 (0),					/* what's a berserker worth? */
	bmaxmvz INIT1 (0),					/* max berserker speed */

	testing INIT1 (0),					/* 1 if testing, else 0 */
	training INIT1 (0),					/* 1 if training game, else 0 */

	jug_likely INIT1 (0),				/* likelihood of a juggernaut appearing */
	black_hole INIT1 (0),				/* NO_BH, LITTLE_BH or BIG_BH */
	is_worm_hole INIT1 (0),				/* 1 if worm_hole, else 0 */
	bh_x INIT1 (0), bh_y INIT1 (0);	/* black hole coordinates */

GLOBAL float norm_fac INIT1 (0.0);	/* scrn size score normalization factor */
GLOBAL u_long score INIT1 (0);		/* score */

GLOBAL chtype death_type;				/* symbol for what ailed you */
GLOBAL chtype grid[MAXX][MAXY];		/* grid for underlying move/score logic */

GLOBAL char orig_cwd[1024];
/*
 *	CURSES STUFF
 */
/*
 * windows
 */
GLOBAL WINDOW *field INIT1(NULL);		/* actual playing field region */
GLOBAL WINDOW *helpline INIT1(NULL);	/* 1-line inplay help window */
GLOBAL WINDOW *helpscr INIT1(NULL);		/* fullscreen inplay help window */
GLOBAL WINDOW *scorewin INIT1(NULL);	/* score info window */
GLOBAL WINDOW *testwin INIT1(NULL);		/* test info window */
/*
 *	eek-specific convenience macros
 *
 *	PUT_LEVEL()	: put level info on screen
 *	PUT_MOVES()	: put moves info on screen
 *	PUT_LEVEL()	: put score info on screen
 *	UPD_LEVEL()	: update level info on screen
 *	UPD_MOVES()	: update moves info on screen
 *	UPD_LEVEL()	: update score info on screen
 */
#define PUT_LEVEL()	PRINT1(scorewin, 0, 2,  "L %3d",  level);
#define PUT_MOVES()	PRINT1(scorewin, 0, 10, "M %4d",  moves);
#define PUT_SCORE(S)	PRINT1(scorewin, 1, 2,  "S %8lu", S);
#define UPD_LEVEL()	PRINT1(scorewin, 0, 4,  "%3d",    level);
#define UPD_MOVES()	PRINT1(scorewin, 0, 12, "%4d",    moves);
#define UPD_SCORE(S)	PRINT1(scorewin, 1, 4,  "%8lu",   S);
#define UPD_LIVES()	PRINT1(scorewin, 1, 18, "%d",     lives);
#define BOX_FIELD()	{ if (need_box) { box(field, (chtype) '|', (chtype) '-'); need_box = 0; } }
/*
 *	MENUS
 */
GLOBAL MenuP main_menu INIT1(NULL);		/* main help menu */
/*
 * COPYRIGHT, ETC.
 */
GLOBAL char *legalstuff INIT1("\n\
COPYRIGHT\n\
\n\
Copyright 1988, 1990, Miles ONeal, Austin, TX, USA.\n\
Copyright 1993, 1997-8 Roadkills-R-Us, Austin, TX, USA.\n\
\n\
All Rights Reserved, except as noted herein.\n\
\n\
This software may be redistributed in any fashion you like, with only\n\
the following limitations.\n\
\n\
   1) You must credit the authors in the source code and accompanying\n\
      documentation.\n\
\n\
   2) You may not use any of the names of the authors or their employers\n\
      in any associated advertising without explicit permission from the\n\
      parties you wish to name.  Finding and contacting them is up to you.\n\
\n\
   3) This copyright must be provided along with the documentation or\n\
      code.\n\
\n\
   4) The accompanying disclaimer must must be provided along with the\n\
      documentation or code.\n\
\n\
If you can actually use this to make money, more power to you.  Just\n\
realize that the eek author is dedicated to keeping good, robust,\n\
useful code freely available to the public, and that philosophy applies\n\
to this package as well.\n\
\n\
\n\
DISCLAIMER\n\
\n\
This software is presented as is, with no warranties expressed\n\
or implied, including implied warranties of merchantability and\n\
fitness.  In no event shall the authors, their institutions, or\n\
any subsequent distributors be liable for any special, direct,\n\
indirect or consequential damages whatsoever resulting from loss\n\
of use, data or profits, whether in an action of contract,\n\
negligence or other tortious action, arising out of or in connection\n\
with the use or performance of this software. In other words, if\n\
you don't like it, don't use it!\n\
\n\
Bugs and feature requests, complaints, and so forth, brought to the eek\n\
author's attention, will be considered.  Any modifications will be made\n\
at the sole discretion of the eek author.\n\
\n\
eek author:\n\
\n\
Miles O'Neal\n\
Roadkills-R-Us\n\
meo@rru.com\n\
\n\
Roadkills-R-Us is a trademark of Miles O'Neal.\n\
\n\
[Non-obscene suggestions for improvement to this copyright and disclaimer\n\
are always welcome.  The intent is to keep control, simply so that nobody\n\
else takes control.  This document should be concise and user-friendly.]\n\
");

#endif /* MAX_ROBOTS */
