/*
 *	play.c - main play routine
 *
 *	Copyright 1988, 1990, Miles ONeal, Austin, TX, USA. All rights reserved.
 *	Copyright 1993, 1997-8 Roadkills-R-Us, Austin, TX, USA. All rights reserved.
 *
 *	Play()			: actually play the game (what a concept)
 *	HelpCleanup()	: cleanup screen after help has been used
 */

#include "eek.h"
#include "Random.h"


static void HelpCleanup(int x, int y);

/*
 *	Play()			: actually play the game (what a concept)
 */
void
Play()
{
	int laststand = 0;
	int x, y, ox, oy;
	int rstat, jstat;
	int inp_char;

/*
 *	create & place ol' Fred.
 */
	do {
		x = MungedRandom(NX-2) + 1;
		y = MungedRandom(NY-2) + 1;
	} while (grid[x][y] && ! InGravityWell(x, y));
	grid[x][y] = HUMANOID;
	Redraw(x, y);

	for (;;) {
/*
 *		Time for Fred (among others) to move.
 *		If he's not acting brave (making a last stand), get the
 *		next move.
 */
		ox = x; oy = y;
		if (laststand == 0) {
			moves++;
			if (0 != madman) {
				inp_char = '-';
				usleep(1000);
			}
			else {
#if ! defined(BSD_CURSES) && defined(REALTIME)
				if (speed > 0) {
					if ((inp_char = getch()) == EOF) {
						sleep(wait_time);
						inp_char = getch();
					}
				} else {
#endif /* ! BSD_CURSES */
					inp_char = getch();
#if ! defined(BSD_CURSES) && defined(REALTIME)
				}
				if ((speed > 0) && (inp_char == EOF))
					inp_char = '5';
#endif /* ! BSD_CURSES */
			}
			if (isupper(inp_char))
			inp_char = tolower(inp_char);
#if defined(KEY_A1)
			else
				switch (inp_char) {
					case KEY_A1 :		inp_char = '7'; break;
					case KEY_UP :		inp_char = '8'; break;
					case KEY_A3 :		inp_char = '9'; break;
					case KEY_LEFT :	inp_char = '4'; break;
					case KEY_B2 :		inp_char = '5'; break;
					case KEY_RIGHT :	inp_char = '6'; break;
					case KEY_C1 :		inp_char = '1'; break;
					case KEY_DOWN :	inp_char = '2'; break;
					case KEY_C3 :		inp_char = '3'; break;
				}
#endif /* KEY_A1 */
			switch (inp_char) {
				case 'y': case '7': if (x > 1) --x; if (y > 1) --y; break;
				case 'u': case '8': if (y > 1) --y; break;
				case 'i': case '9': if (x < NX-2) x++; if (y > 1) --y; break;
				case 'h': case '4': if (x > 1) --x; break;
				case 'j': case '5': case '0': case ' ': break;
				case 'k': case '6': if (x < NX-2) x++; break;
				case 'n': case '1': if (x > 1) --x; if (y < NY-2) y++; break;
				case 'm': case '2': if (y < NY-2) y++; break;
				case ',': case '3': if (x<NX-2) x++; if (y < NY-2) y++; break;
				case 't': case '-': Teleport(&x, &y, 0); break;
				case 's': case '.': laststand++; break;
				case 'q': refresh(); LevelExit(WIMPED_OUT); break;
				case '\033': refresh(); LevelExit(ESCAPED); break;
				case '?': OneLineHelp(x, y); HelpCleanup(x, y); continue;
				case '!': case '\014': ForceRefreshBF(x, y); continue;
				default:	continue;
			}
		}
		grid[ox][oy] = EOS;
		PUT_CH(field, oy, ox, ' ');
/*
 *		Did Fred step on the wrong toes or get underfoot?
 *		See what happens - somebody always loses (usually
 *		Fred).  Everybody loses to a Black Hole...
 */
		if (grid[x][y])
			if (Random(3))
				Munch(x, y, grid[x][y]);
		if (InGravityWell(x, y))
			if (is_worm_hole)
				Teleport(&x, &y, 1);
			else
				Munch(x, y, BLACK_HOLE);
/*
 *		No? very good, Fred.  Put Fred back on the map.
 */
		grid[x][y] = HUMANOID;
		PUT_CH_SO(field, y, x, HUMANOID);
		wrefresh(field);
/*
 *		Now let everybody else do their thang.  If Fred
 *		survived, refigure the score.  Otherwise, say
 *		"bye bye!" (unless Fred is in training).
 */
		rstat = MoveRobots(x, y);
		jstat = ProcJuggernaut(x, y);
		if (rstat && jstat) {
			ScoreLevel(ALIVE);
			LevelExit(SURVIVED);
			break;
		} else {
			if (testing)
				UpdateTestwin(x, y);
			ScoreMove();
			if (training) {
				grid[x][y] = HUMANOID;
				PUT_CH_SO(field, y, x, HUMANOID);
				wmove(field, 0, 0);
				wrefresh(field);
			}
		}
	}
}


/*
 *	HelpCleanup()	: cleanup screen after help has been used
 */
static void
HelpCleanup(int x, int y)
{
	need_box = 1;
	if (NY != SY + 2) {
		move(NY - 1, 0);
		wclrtoeol(stdscr);
		refresh();
	}
	Redraw(x, y);
	wmove(scorewin, 1, 0);
	wclrtoeol(scorewin);
	PUT_SCORE(score);
	wrefresh(scorewin);
}
