/*
 *	redraw.c 	: drawing routines
 *
 *	Copyright 1988, 1990, Miles ONeal, Austin, TX, USA. All rights reserved.
 *	Copyright 1993, 1997-8 Roadkills-R-Us, Austin, TX, USA. All rights reserved.
 *
 *	Redraw()				: redraw the screen efficiently
 *	ForceRefresh()		: force a complete screen refresh
 *	ForceRefreshBF()	: force a complete screen refresh (with bugfix)
 *	DrawScorewin()		: force a complete scorewin refresh
 *	UpdateScorewin()	: force a complete scorewin update
 *	UpdateTestwin()	: update test window variables
 */

#include "eek.h"

/*
 *	Redraw()				: redraw the screen efficiently
 */
Redraw(int ix, int iy)
{
	/* register */ int x, y;
	static int first_time = 1;
/*
 *	instructions, scoring, etc - drawn first time only.
 */
	if (first_time) {
		clear();
/*
 *		basic movement map - main keyboard and keypad.
 */
		PUT_STR_SO(stdscr, 0, NX+8, "MOVES");
		PUT_STR(stdscr, 1, NX+2, "y u i");
		PUT_STR(stdscr, 2, NX+2, " \\|/");
		PUT_STR(stdscr, 3, NX+2, "h-j-k");
		PUT_STR(stdscr, 4, NX+2, " /|\\");
		PUT_STR(stdscr, 5, NX+2, "n m ,");

		PUT_STR(stdscr, 1, NX+14, "7 8 9");
		PUT_STR(stdscr, 2, NX+14, " \\|/");
		PUT_STR(stdscr, 3, NX+14, "4-5-6");
		PUT_STR(stdscr, 4, NX+14, " /|\\");
		PUT_STR(stdscr, 5, NX+14, "1 2 3");
		if (!testing) {
/*
 *			Other commands the puppeteer probably cares about
 */
			PUT_STR_SO(stdscr, 7, NX+6, "COMMANDS");
			PUT_CH_SO(stdscr, 8, NX+2, 't');
			PUT_CH_SO(stdscr, 8, NX+4, '-');
			PUT_STR(stdscr, 8, NX+6, "teleport");
			PUT_CH_SO(stdscr, 9, NX+2, 's');
			PUT_CH_SO(stdscr, 9, NX+4, '.');
			PUT_STR(stdscr, 9, NX+6, "last stand");
			PUT_CH_SO(stdscr, 10, NX+2, 'q');
			PUT_STR(stdscr, 10, NX+6, "quit");
			PUT_CH_SO(stdscr, 11, NX+2, '?');
			PUT_STR(stdscr, 11, NX+6, "help");
			PUT_STR_SO(stdscr, 12, NX+2, "^L");
			PUT_STR(stdscr, 12, NX+6, "redraw");
/*
 *			Legend to what's on the field.
 */
			PUT_STR_SO(stdscr, 14, NX+9, "KEY");
			PUT_CH_SO(stdscr, 15, NX+2, 'I');
			PUT_STR(stdscr, 15, NX+4, "You");
			PUT_CH_SO(stdscr, 15, NX+12, '#');
			PUT_STR(stdscr, 15, NX+14, "Fence");
			PUT_CH_SO(stdscr, 16, NX+2, '=');
			PUT_STR(stdscr, 16, NX+4, "Robot");
			PUT_CH_SO(stdscr, 16, NX+12, '@');
			PUT_STR(stdscr, 16, NX+14, "Junk");
			PUT_CH_SO(stdscr, 17, NX+2, 'X');
			PUT_STR(stdscr, 17, NX+4, "Berserker");
			PUT_CH_SO(stdscr, 18, NX+2, '^');
			PUT_STR(stdscr, 18, NX+4, "Berserker Trap");
			PUT_CH_SO(stdscr, 19, NX+2, '*');
			PUT_STR(stdscr, 19, NX+4, "Juggernaut");
			PUT_CH_SO(stdscr, 20, NX+2, '.');
			PUT_STR(stdscr, 20, NX+4, "Black Hole");
		} else {
			PUT_STR(testwin, 0,  0, "nrobots = %3d");
			PUT_STR(testwin, 1,  0, "rob_pts = %4d");
			PUT_STR(testwin, 2,  0, "nberserk = %3d");
			PUT_STR(testwin, 3,  0, "bmaxmvz = %3d");
			PUT_STR(testwin, 4,  0, "ber_pts = %7d");
			PUT_STR(testwin, 5,  0, "max_traps = %3d");
			PUT_STR(testwin, 6,  0, "nfences = %3d");
			PUT_STR(testwin, 7,  0, "nice_lvl = %4d");
			PUT_STR(testwin, 8,  0, "nice_dist = %3d");
			PUT_STR(testwin, 9,  0, "jug_likely = %3d");
			PUT_STR(testwin, 10, 0, "black_hole = %3d");
			PUT_STR(testwin, 11, 0, "_worm_hole = %3d");
			PUT_STR(testwin, 12, 0, "norm_fac = %5.3f");
			PUT_STR(testwin, 13, 0, "tele_tries = %5d");
		}
		refresh();
/*
 *		current level, score, & # moves so far this level
 */
		DrawScorewin(score);
		first_time = 0;
	} else {
		wclear(field);
	}
/*
 *	playing field	: 1st the border, then the players & objects
 */
	BOX_FIELD();
/*
 *	draw the players
 */
	for (y = 1; y < NY-1; y++) {
		for (x = 1; x < NX-1; x++) {
			if (grid[x][y]) {
				if (grid[x][y] != HUMANOID) {
					PUT_CH(field, y, x, grid[x][y]);
				} else {
					PUT_CH_SO(field, iy, ix, HUMANOID);
				}
			}
		}
	}
	BOX_FIELD();
	wmove(field, 0, 0);
	wrefresh(field);
	if (new_level) {
		new_level--;
		UPD_LEVEL();
	}
	UPD_MOVES();
	if (testing)
		UpdateTestwin(ix, iy);
	wrefresh(scorewin);
}


/*
 *	ForceRefresh()		: force a complete screen refresh
 */
void
ForceRefresh()
{
	if (stdscr) {
		touchwin(stdscr);
		wrefresh(stdscr);
		if (testwin) {
			touchwin(testwin);
			wrefresh(testwin);
		}
		if (field) {
			touchwin(field);
			wrefresh(field);
		}
		if (scorewin) {
			touchwin(scorewin);
			wrefresh(scorewin);
		}
	}
}


/*
 *	ForceRefreshBF()	: force a complete screen refresh (with bugfix)
 *
 *		Use this when ForceRefresh() loses Fred
 */
void
ForceRefreshBF(int x, int y)
{
	if (stdscr) {
		touchwin(stdscr);
		wrefresh(stdscr);
		if (testwin) {
			touchwin(testwin);
			wrefresh(testwin);
		}
		if (field) {
			touchwin(field);
#if defined(linux)
			PUT_CH(field, y, x, HUMANOID);
#else
			PUT_CH_SO(field, y, x, HUMANOID);
#endif
			wrefresh(field);
		}
		if (scorewin) {
			touchwin(scorewin);
			wrefresh(scorewin);
		}
	}
}


/*
 * DrawScorewin()		: force a complete scorewin refresh
 */
void
DrawScorewin(long score)
{
	PUT_LEVEL();
	PUT_MOVES();
	PUT_SCORE(score);
	UPD_LIVES();
	wrefresh(scorewin);
}


/*
 * UpdateScorewin()	: force a complete scorewin update
 */
void
UpdateScorewin(long score)
{
	UPD_LEVEL();
	UPD_MOVES();
	UPD_SCORE(score);
	UPD_LIVES();
	wrefresh(scorewin);
}



/*
 *	UpdateTestwin()	: update test window variables
 */

void
UpdateTestwin(int ix, int iy)
{

	PRINT1(testwin, 0,  10, "%3d",   nrobots);
	PRINT1(testwin, 1,  10, "%4d",   rob_pts);
	PRINT1(testwin, 2,  11, "%3d",   nberserk);
	PRINT1(testwin, 3,  10, "%3d",   bmaxmvz);
	PRINT1(testwin, 4,  10, "%7d",   ber_pts);
	PRINT1(testwin, 5,  12, "%3d",   MAX_TRAPS);
	PRINT1(testwin, 6,  10, "%3d",   nfences);
	PRINT1(testwin, 7,  11, "%4d",   nice_lvl);
	PRINT1(testwin, 8,  12, "%3d",   nice_dist);
	PRINT1(testwin, 9,  13, "%3d",   jug_likely);
	PRINT1(testwin, 10, 13, "%3d",   black_hole);
	PRINT1(testwin, 11, 13, "%3d",   is_worm_hole);
	PRINT1(testwin, 12, 11, "%5.3f", norm_fac);
	PRINT1(testwin, 13, 13, "%5d",   tele_tries);
	wmove(testwin, 14, 0); wprintw(testwin, "Fred@y=%3d x=%3d", iy, ix);
	wrefresh(testwin);
}
