/*
 *	robots.c - robot & related stuff handling
 *
 *	Copyright 1988, 1990, Miles ONeal, Austin, TX, USA. All rights reserved.
 *	Copyright 1993, 1997-8 Roadkills-R-Us, Austin, TX, USA. All rights reserved.
 *
 *	MoveRobots()				: move robots, etc
 *	CreateThing()				: create & place a simple thing
 *	CreateRobots()				: create robots
 *	CreateBerserkers()		: create berserkers
 *	CreateBerserkerTraps()	: create berserker traps
 *	CreateFences()				: create fences
 */

#include "eek.h"
#include "Random.h"


MoveRobots(ix, iy)
{
	register int x, y;
	int tx, ty, mvz;
	int l[MAXX][MAXY];
	int flag;

/*
 *	init scoring grid 
 */
	for (y = NY-1; y > 0; y--)
		for (x = NX-1; x > 0; x--)
			l[x][y] = 0;
/*
 *	wander thru grid. when you find a robotic entity,
 *	move them towards the humanoid.
 */
	for (y = NY-1; y > 0; y--) {
		for (x = NX-1; x > 0; x--) {
			tx = x;
			ty = y;
			switch (grid[x][y]) {
/*
 *			basic robots move 1 location towards the humanoid.
 *			they will fall into black holes and disappear.
 */
			case ROBOT:
				if (tx < ix) tx++;
				if (tx > ix) --tx;
				if (ty < iy) ty++;
				if (ty > iy) --ty;
				if (InGravityWell(tx, ty))  /* never survive wormholes */
					l[tx][ty] = EOS;
				else
					l[tx][ty]++;
				break;
/*
 *			berserkers move 1 or more location[s] towards the humanoid.
 *			they will fall into black holes and disappear. they also
 *			get caught by berserker traps.
 */
			case BERSERKER:
				for (mvz = 1; mvz <= bmaxmvz; mvz++) {
					if (tx < ix) tx++;
					if (tx > ix) --tx;
					if (ty < iy) ty++;
					if (ty > iy) --ty;
					if (tx == ix && ty == iy) {
						Munch(ix, iy, BERSERKER);
					} else if (grid[tx][ty] == TRAP) {
						l[tx][ty] = -500;
						break;
					} else if (InGravityWell(tx, ty)) {
						if (is_worm_hole) {  /* they have a slight chance of surviving */
							Teleport(&ty, &tx, 0);
							if (grid[tx][ty] == TRAP) {
								l[tx][ty] = -500;
								break;
							}
						} else {
							l[tx][ty] = -500;
							break;
						}
					} else if (l[tx][ty]) {
						l[tx][ty] = 2;
					}
				}
				if (l[tx][ty] != -500) {
					l[tx][ty] = 10000;
					grid[tx][ty] = EOS;
				}
				break;
/*
 *	these next items don't move.
 */
			case JUNK:
				l[x][y] = 2;
				break;
			case FENCE:
				l[x][y] = -10000;
				break;
			case TRAP:
				l[x][y] = -500;
				break;
			} /* switch on grid[x][y] */
		} /* each x */
	} /* each y */
/*
 *	NOW that we know what's where, let's figure out who's still
 *	around to hassle the poor humanoid.
 */
	flag = 0;
	for (y = NY-1; y > 0; y--) {
		for (x = NX-1; x > 0; x--) {
			if (l[x][y] > 0) {
/*
 *				exactly 1 robot is here
 */
				if (l[x][y] == 1) {
					flag++;
					grid[x][y] = ROBOT;
					PUT_CH(field, y, x, ROBOT);
/*
 *				at least 1 berserker is here. exactly 1 will survive.
 */
				} else if (l[x][y] >= 10000){
					flag++;
					grid[x][y] = BERSERKER;
					PUT_CH(field, y, x, BERSERKER);
/*
 *				some nasty robotoid collision occurred from which there were
 *				no survivors.
 */
				} else {
					grid[x][y] = JUNK;
					PUT_CH(field, y, x, JUNK);
				}
/*
 *				OOPS. the humanoid was in an occupied stall.
 *				somebody dies.
 */
				if (x == ix && y == iy)
					Munch(ix, iy, grid[x][y]);
/*
 *			did some animated bucket of bolts slip through the
 *			cracks? if so, TOO BAD. implode its power cells.
 */
			} else if (l[x][y] == 0) {
				if (grid[x][y] == ROBOT || grid[x][y] == BERSERKER) {
					grid[x][y] = EOS;
					PUT_CH(field, y, x, ' ');
				}
			}
		} /* each x */
	} /* each y */
	wmove(field, 0, 0);
	wrefresh(field);
	return (!flag);
}


/*
 *	create & place a simple thing
 */
void
CreateThing(
#ifdef __STDC__
chtype thing)
#else
thing)
chtype thing;
#endif

{
	int x, y;
	int dist = 2;
	int dist2;

	if (thing == TRAP)
		dist += (bmaxmvz << 1);
		dist2 = dist >> 1;
	do {
		x = MungedRandom(NX - dist) + dist2;
		y = MungedRandom(NY - dist) + dist2;
	} while (grid[x][y]);
	grid[x][y] = thing;
}


/*
 *	create robots
 */
void
CreateRobots()
{
	/* register */ int i;

	if (level > 1 && nrobots <= maxrobots)
		nrobots += (float) (Random(3) + 2) / norm_fac;
	if (level > 1)
		rob_pts += (((level / 5) * 2) + 1);
	for (i = nrobots; i; i--)
		CreateThing(ROBOT);
}


/*
 *	create berserkers
 */
void
CreateBerserkers()
{
	/* register */ int i;

	if (level > 1 && nberserk <= maxrobots)
		nberserk += Random(1 + Random((level < 10) ? level : level / 2));
	if (nberserk > 0) {
		bmaxmvz = 1 + Random(1 + ((level % 3) ? Random(3) :(level / 6)));
		bmaxmvz = MIN(bmaxmvz, (MIN(NX, NY) / 10));
		if (level > 1)
				ber_pts = level * bmaxmvz * 100;
		for (i = nberserk; i; i--)
			CreateThing(BERSERKER);
	}
}


/*
 *	create berserker traps
 */
void
CreateBerserkerTraps()
{
	/* register */ int i;
	int max_traps = MAX_TRAPS;

	for (i = max_traps; i; i--)
		CreateThing(TRAP);
}


/*
 *	create fences
 */
void
CreateFences()
{
	/* register */ int i;

	if (nfences > minfences)
		if (! Random(2))
			nfences--;
	for (i = nfences; i; i--)
		CreateThing(FENCE);
}
